#!/bin/csh -f
#
# Usage: install.csh mc-tinker_path amber_path
#
# C shell script used to compile a serial version of AMBERPLUS
#
if ($#argv != 2) then
   echo "Usage: install.csh mc-tinker_path amber_path"
   echo "For example: install.csh /usr/mcsi-2010-1 /usr/amber11 "
   exit 1
endif
#
set mctdir = $argv[1]
set ambdir = $argv[2]
set ampdir = `pwd`
#
set tksdir = $mctdir/tinker/source
set mcsdir = $mctdir/srcmct
set amsdir = $ambdir/src
set apsdir = $ampdir/src
#
set snddir = $amsdir/sander
set amcdir = $amsdir/mcsi
set exedir = $ambdir/exe
set sqmdir = $ambdir/AmberTools/src/sqm/
#
# Make mc-tinker directory in the AMBER src directory
#
mkdir $amcdir
#####################################################################
#
# Copy TINKER subroutines
#
cd $tksdir
#
cat mechanic.f nextarg.f upcase.f trimtext.f analysis.f \
version.f prtxyz.f gradient.f hessian.f \
freeunit.f promo.f precise.f command.f \
readxyz.f attach.f active.f cluster.f bonds.f \
angles.f torsions.f cutoffs.f rings.f katom.f orbital.f kbond.f \
kangle.f kstrbnd.f kurey.f kangang.f kopbend.f kimprop.f kimptor.f \
ktors.f kstrtor.f kvdw.f kcharge.f kdipole.f kmpole.f kpolar.f solvate.f \
korbit.f molecule.f unitcell.f lattice.f restrain.f smooth.f mutate.f \
fatal.f bounds.f ebond3.f eangle3.f estrbnd3.f eurey3.f eangang3.f \
eopbend3.f eimprop3.f eimptor3.f etors3.f estrtor3.f ebuck3.f elj3.f ehal3.f \
egauss3.f echarge9.f echarge3.f echgdpl3.f edipole3.f empole3.f erxnfld3.f  \
esolv3.f egeom3.f extra3.f jacobi.f lowcase.f gettext.f born.f solve.f \
search.f ebond1.f eangle1.f estrbnd1.f eurey1.f eangang1.f \
eopbend1.f eimprop1.f ebuck2.f elj2.f erxnfld2.f precond.f lights.f empole1.f \
eimptor1.f etors1.f estrtor1.f ebuck5.f elj5.f ehal5.f ebuck1.f elj1.f \
ehal1.f egauss1.f echarge7.f echarge5.f echarge1.f echgdpl1.f edipole1.f \
erxnfld1.f esolv1.f egeom1.f extra1.f ebond2.f eangle2.f estrbnd2.f eurey2.f \
eangang2.f eopbend2.f eimprop2.f eimptor2.f etors2.f estrtor2.f \
ehal2.f egauss2.f echarge8.f echarge2.f echgdpl2.f edipole2.f empole2.f \
esolv2.f egeom2.f extra2.f numeral.f prtint.f ebond.f eangle.f estrbnd.f \
eurey.f eangang.f eopbend.f eimprop.f eimptor.f etors.f estrtor.f ebuck4.f \
elj4.f ehal4.f ebuck.f elj.f ehal.f egauss.f echarge6.f echarge4.f echarge.f \
echgdpl.f edipole.f empole.f erxnfld.f esolv.f egeom.f extra.f \
control.f nexttext.f getword.f sort.f getnumb.f \
prmkey.f getstring.f piplane.f number.f \
torphase.f overlap.f pitilt.f pialter.f groups.f switch.f \
image.f rotpole.f induce.f surface.f surfatom.f \
readprm.f pimove.f prterr.f cholesky.f column.f > $amcdir/tinker.f
#
mv sizes.i sizes.i.bak
cp *.i $amcdir
mv sizes.i.bak sizes.i
#
# Change to mcsi directory
#
cd $amcdir
#
# Copy MCSI include files
#
cp $mcsdir/sizes.i sizes.i
cp $mcsdir/mcparm.i mcparm.i
cp $mcsdir/mcperc.inc mcperc.inc
cp $mcsdir/mcparm.inc mcparm.inc
cp $mcsdir/mccomm.inc mccomm.inc
cp $mcsdir/eemcmm.inc eemcmm.inc
#
# Copy MCSI files
#
cp $mcsdir/mcsrc1.f mcsrc1.f
cp $mcsdir/mcsrc2.f mcsrc2.f
cp $mcsdir/mcsrc3.f mcsrc3.f
cp $mcsdir/mcsrc4.f mcsrc4.f
cp $mcsdir/mcsrc5.f mcsrc5.f
cp $mcsdir/mcsrc6.f mcsrc6.f
cp $mcsdir/mcsrc7.f mcsrc7.f
cp $mcsdir/mcsrc8.f mcsrc8.f
cp $mcsdir/emsrc1.f emsrc1.f
cp $mcsdir/getkey.f getkey.f
cp $mcsdir/energ.f energ.f
cp $mcsdir/grads.f grads.f
cp $mcsdir/hesss.f hesss.f
cp $mcsdir/field.f field.f
cp $mcsdir/piscf.f piscf.f
cp $mcsdir/initial.f initial.f
cp $mcsdir/readfile.f readfile.f
#
# Activate MCSI files
#
foreach file (emsrc1 mcsrc1 mcsrc5 mcsrc6 mcsrc8 readfile tinker)
    sed -e 's/CAMBP//g' $file.f > $file.f.tmp
    mv $file.f.tmp $file.f
end
#
# Copy AMBERPLUS files
#
cp $apsdir/Makefile.mcsi Makefile
cp $apsdir/getprm.f getprm.f
cp $apsdir/mcinter2.f mcinter2.f
cp $apsdir/extra.h extra.h
cp $apsdir/parallel.h parallel.h
#
# Change the name of common blocks in tinker.f 
# to avoide interference with those in sander subroutines.
#
foreach file ( *.i )
   sed 's/common \//common\/t/' $file > $file.tmp
   mv $file.tmp $file
end
#
# Change the names of some subroutines in tinker.f, getkey.f and piscf.f
# to avoide interference with those in sander subroutines.
#
sed 's/unitcell/unitcell_t/' tinker.f > tinker.f.tmp
mv tinker.f.tmp tinker.f
sed 's/jacobi/jacobi_t/' tinker.f > tinker.f.tmp
mv tinker.f.tmp tinker.f
sed 's/upcase/upcase_t/' tinker.f > tinker.f.tmp
mv tinker.f.tmp tinker.f
sed 's/sort2/sort2_t/' tinker.f > tinker.f.tmp
mv tinker.f.tmp tinker.f
sed 's/molecule/molecule_t/' tinker.f > tinker.f.tmp
mv tinker.f.tmp tinker.f

sed 's/upcase/upcase_t/' getkey.f > getkey.f.tmp
mv getkey.f.tmp getkey.f
sed 's/jacobi/jacobi_t/' piscf.f > piscf.f.tmp
mv piscf.f.tmp piscf.f

#
# Change 'fu5' parameter in mcparm.inc 
#
sed 's/FU5 = 5/FU5 = 85/' mcparm.inc > mcparm.inc.tmp
mv mcparm.inc.tmp mcparm.inc

#
# Change to sqm directory
#
cd $sqmdir
#
# Backup and patch sqm files
#
cp qm2_read_nm_and_alloc.f qm2_read_nm_and_alloc.f.org
cp $apsdir/qm2_read_nm_and_alloc.patch .
patch -p0 -N -r qm2_read_nm_and_alloc.reject < qm2_read_nm_and_alloc.patch

cp qm_link_atoms.f qm_link_atoms.f.org
cp $apsdir/qm_link_atoms.patch .
patch -p0 -N -r qm_link_atoms.reject < qm_link_atoms.patch

cp qmmm_module.f qmmm_module.f.org
cp $apsdir/qmmm_module.patch .
patch -p0 -N -r qmmm_module.reject < qmmm_module.patch

cp qmmm_qmtheorymodule.f qmmm_qmtheorymodule.f.org
cp $apsdir/qmmm_qmtheorymodule.patch .
patch -p0 -N -r qmmm_qmtheorymodule.reject < qmmm_qmtheorymodule.patch

cp Makefile Makefile.org
cp $apsdir/Makefile.sqm.patch .
patch -p0 -N -r Makefile.reject < Makefile.sqm.patch

cp makedepend makedepend.org
cp $apsdir/makedepend.sqm.patch .
patch -p0 -N -r makedepend.reject < makedepend.sqm.patch
#
# Make depend file
#
make depend
#
# Change to sander directory
#
cd $snddir
#
# Backup and patch sander files
#
cp def_time.h def_time.h.org 
cp $apsdir/def_time.patch .
patch -p0 -N -r def_time.reject < def_time.patch

cp dynlib.f dynlib.f.org 
cp $apsdir/dynlib.patch .
patch -p0 -N -r dynlib.reject < dynlib.patch

cp ew_setup.f ew_setup.f.org 
cp $apsdir/ew_setup.patch .
patch -p0 -N -r ew_setup.reject < ew_setup.patch

cp force.f force.f.org 
cp $apsdir/force.patch .
patch -p0 -N -r force.reject < force.patch

cp mdfil.f mdfil.f.org 
cp $apsdir/mdfil.patch .
patch -p0 -N -r mdfil.reject < mdfil.patch

cp mdread.f mdread.f.org 
cp $apsdir/mdread.patch .
patch -p0 -N -r mdread.reject < mdread.patch

cp new_time.f new_time.f.org 
cp $apsdir/new_time.patch .
patch -p0 -N -r new_time.reject < new_time.patch

cp parallel.f parallel.f.org 
cp $apsdir/parallel.patch .
patch -p0 -N -r parallel.reject < parallel.patch

cp printe.f printe.f.org 
cp $apsdir/printe.patch .
patch -p0 -N -r printe.reject < printe.patch

cp runmd.f runmd.f.org 
cp $apsdir/runmd.patch .
patch -p0 -N -r runmd.reject < runmd.patch

cp sander.f sander.f.org 
cp $apsdir/sander.patch .
patch -p0 -N -r sander.reject < sander.patch

cp short_ene.f short_ene.f.org 
cp $apsdir/short_ene.patch .
patch -p0 -N -r short_ene.reject < short_ene.patch

cp Makefile Makefile.org
cp $apsdir/Makefile.sander.patch .
patch -p0 -N -r Makefile.reject < Makefile.sander.patch

cp makedepend makedepend.org
cp $apsdir/makedepend.sander.patch .
patch -p0 -N -r makedepend.reject < makedepend.sander.patch

#
# Copy AMBERPLUS files
#
cp $apsdir/mcinter1.f mcinter1.f
cp $apsdir/taper.f taper.f

#
# Make sanderplus
#
make clean
make depend
make sanderplus
#
# mv sanderplus to exe directory
#
mv sanderplus $exedir/sanderplus
