c
c                               MC-TINKER
c
c     PROGRAM FOR MULTICONFIGURATION MOLECULAR MECHANICS CALCULATIONS
c
      subroutine mctsetup
c
c     This routine is setup for MCMM calculations
c
      use qmmm_module
      use mcinter
c
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
      include 'extra.h'
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      character*256 finame
      character*7  fistat
      character*20  fstat
      logical lexit
c
c     set imcatm default value
c
      do 100 i=1,qmmm_struct%nquant_nlink
         imcatm(i)=qmmm_nml%iqmatoms(i)
 100  continue
c
c     Open the input and output files (fu5 and fu6)
c
c     Use the character variables fistat (file status), finame (filename),
c     and the subprogram mcfope to open the input data file and the long
c     output files which are linked to FORTRAN units fu5 and fu6, 
c     respectively.
c
c     The error statement for units fu5 and fu6 are written to unit 6
c     (assumed to be standard out) because if there is a problem
c     with units fu5 or fu6 then the code cannot use fu6 for its output
c     and the following error message may not be written.
c
c      lexit  = .false.
c
c      fistat = 'unknown'
c      finame = 'esp.fu86'
c
c      call mcfope (fu6, fistat, finame, lexit)
c      if (lexit) then
c         write (6, *) 'Error opening the long output file'
c         stop 'main 1'
c      endif
c
      if(master) then
c
         lexit  = .false.
c
         fistat = 'old'
         finame = esp85
c
         call mcfope (fu5, fistat, finame, lexit)
c
         if (lexit) then
            write (6, *) 'Error opening the input file'
            stop 'mctsetup 1'
         endif
c
c     Write out program header to unit fu6
c
         call mcpthd(6)
c
      end if
c
c     Read in the options from the general input file (fu5)
c
      call mcrfgi
c
c
      if(master) then
c
c     Close the input data file
c
         fstat = 'keep'
         call mcfclo (fu5, fstat)
c
c     Write out the summary table of all input parameters
c
         call mcptab
c
      end if
c
c     Set up the calculation by reading and preparing the appropriate 
c     information
c
      call mcscal
c
c     file open for charges or electrostatic potentials output
c
      if(master) then
         if(nstpchg.ne.0) then
            if(ifmtchg.eq.1) then
               open(iochg,file=mcchg,form='formatted')
            else
               open(iochg,file=mcchg,form='unformatted')
            end if
         end if
c
         if(nstpelp.ne.0) then
            if(ifmtelp.eq.1) then
               open(ioelp,file=mcelp,form='formatted')
            else
               open(ioelp,file=mcelp,form='unformatted')
            end if
         end if
c
         if(nstpene.ne.0) then
            if(ifmtene.eq.1) then
               open(ioene,file=mcene,form='formatted')
            else
               open(ioene,file=mcene,form='unformatted')
            end if
         end if
      end if
c
      return
c
      end
c
c ***************************************************************************
c     mccalc
c ***************************************************************************
c
      subroutine mccalc(coords,frc,emcmm,natom,periodic)
c
c     This subroutine is the driver for the actual MCMM calculation.
c     (Titus V Albu January 2002)
c
      use qmmm_module
      use constants
      use mcinter
c
      implicit double precision (a-h,o-z)
c
c
#ifdef MPI
c     =========================== AMBER/MPI ===========================
#     ifdef MPI_DOUBLE_PRECISION
#       undef MPI_DOUBLE_PRECISION
#     endif
      include 'mpif.h'
#     ifdef CRAY_PVP
#       define MPI_DOUBLE_PRECISION MPI_REAL8
#     endif
c     ========================= END AMBER/MPI =========================
#endif
c
#     include "parallel.h"
#     include "extra.h"
c
c
#     include "../sander/def_time.h"
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      include 'eemcmm.inc'                                              0831MH07 
c
      integer periodic,qm_no
      dimension coords(3*natom),frc(3*natom)
      dimension fmcmm(3*natom)
      dimension bxbnd(2,3)
c
      dimension x(mcmcar), dx(mcmcar), f(mcmcar,mcmcar)
      dimension phi(mcmatm)                                             0831MH07 
#ifdef MPI
      dimension phitmp(mcmatm),frctmp(3*natom)
#endif
c
      dimension frcmod(3),dlink(3)
c
      character*7  fistat
      character*20 fstat
      character*80 word(40)
c
      logical lexit
c
c     Initialize some variables
c
      mcncar = 3*mcnatm
c
      do i=1,mcmcar
         x(i)=0.d0
         dx(i)=0.d0
         do j=1,mcmcar
            f(i,j)=0.d0
         enddo
      enddo
c
      do i=1,3*natom
         fmcmm(i)=0.0d+00
      end do
c
      do i=1,qmmm_struct%nquant_nlink
         qmmm_struct%dxyzqm(1,i)=0.0d+00
         qmmm_struct%dxyzqm(2,i)=0.0d+00
         qmmm_struct%dxyzqm(3,i)=0.0d+00
      end do
c
c     generate link atom coordinates
c
      if(qmmm_struct%nlink.gt.0) call adj_mm_link_pair_crd(coords)
c
c     calculate electrostatic potentials on QM atoms
c
      if(mciee.eq.1) then
c
         call timer_start(TIME_EEMCMMESP)
c
         do i=1,mcnatm
            phi(i)=0.0d+00
         end do
c
         call mccphi(coords,phi,natom,mcmatm,periodic)
c
#ifdef MPI
         call mpi_allreduce(phi,phitmp,mcnatm,
     &        mpi_double_precision,mpi_sum,commsander,ierr)
c
         do i=1,mcnatm
            phi(i)=phitmp(i)
         end do
#endif
c
c     copy phi to phir
c
         do i=1,mcnatm
            phir(i)=phi(i)
         end do
c
         call timer_stop(TIME_EEMCMMESP)
c
      end if
c
c     copy qmmm_struct%qm_coords to x
c
      do i=1,qmmm_struct%nquant_nlink
         j=imcatm(i)
         x(3*i-2)=coords(3*j-2)/BOHRS_TO_A
         x(3*i-1)=coords(3*j-1)/BOHRS_TO_A
         x(3*i  )=coords(3*j  )/BOHRS_TO_A
c$$$         if(master) then
c$$$            write(6,'(a6,1x,i3,4f15.8)') "master",i,coords(3*j-2),
c$$$     &           coords(3*j-1),coords(3*j),phi(i)*27.21134D+00
c$$$         else
c$$$            write(*,'(a6,1x,i3,4f15.8)') "slave",i,coords(3*j-2),
c$$$     &           coords(3*j-1),coords(3*j),phi(i)*27.21134D+00
c$$$         end if
      end do
c                                           
c     Transform the potential to a.u.       
c                                           
c      if(mciee.eq.1 .and. mciuph.eq.1) then 
c         do i=1,mcncar                         
c            phi(i)=phi(i)/fhaev          
c         end do                           
c      end if                              
c
c     Determine what type of calculation and call the apropriate routine
c
      if(mciee.eq.0) then
c
c         if (mcityp.eq.1) then
c            call mehook(x,v,e1,e2,e12,mcmcar,mciord,icomplex)
c         else if (mcityp.eq.2) then
            call mghook(x,v,e1,e2,e12,dx,mcmcar,mciord,icomplex)
c         else
c            call mhhook(x,v,e1,e2,e12,dx,f,mcmcar,mciord,icomplex)
c         endif
c
      else
c
c         if (mcityp.eq.1) then
c            call eeehok(x,phi,v,e1,e2,e12)
c         else if (mcityp.eq.2) then
            call eeghok(x,phi,v,e1,e2,e12,dx)
c         else
c            call eehhok(x,phi,v,e1,e2,e12,dx,f)
c         endif
c
      end if
c
c     Print the geometry of the point and the MCMM results 
c
c      call mcprez(mcipts,x,v,e1,e2,e12,dx,f)
c
c      if(mciee.eq.1) call eeprez(phi,v)
c
c     copy calculated energy to emcmm
c
c      emcmm=fhakc*v
      emcmm=AU_TO_KCAL*v/dble(numtasks)
c
c     copy calculated hamiltonian matrix and energy
c
      vmc11=e1
      vmc22=e2
      vmc12=e12
      vmceig=v
c
c     calculate QM-MM gradient
c
      if(mciee.eq.1) then
         call timer_start(TIME_EEMCMMESP)
         call mcigrd(coords,fmcmm,natom,periodic)
         call timer_stop(TIME_EEMCMMESP)
c
#ifdef MPI
         call mpi_allreduce(fmcmm,frctmp,3*natom,
     &        mpi_double_precision,mpi_sum,commsander,ierr)
c
         do i=1,3*natom
            fmcmm(i)=frctmp(i)
         end do
c
         call mpi_allreduce(qmmm_struct%dxyzqm,frctmp,
     &        3*qmmm_struct%nquant_nlink,mpi_double_precision,
     &        mpi_sum,commsander,ierr)
c
         do i=1,qmmm_struct%nquant_nlink
            qmmm_struct%dxyzqm(1,i)=frctmp(3*i-2)
            qmmm_struct%dxyzqm(2,i)=frctmp(3*i-1)
            qmmm_struct%dxyzqm(3,i)=frctmp(3*i)
c$$$         if(master) then
c$$$            write(6,'(a6,1x,i3,4f15.8)') "master",i,frctmp(3*i-2),
c$$$     &           frctmp(3*i-1),frctmp(3*i)
c$$$         else
c$$$            write(*,'(a6,1x,i3,4f15.8)') "slave ",i,frctmp(3*i-2),
c$$$     &           frctmp(3*i-1),frctmp(3*i)
c$$$         end if
         end do
c
c         if(master) call frccheck(frctmp,natom)
#endif
c
      end if
c
c     add QM forces to qmmm_struct%dxyzqm
c
      ftmp=AU_TO_KCAL/BOHRS_TO_A
c
      do i=1,qmmm_struct%nquant_nlink
         qmmm_struct%dxyzqm(1,i)=qmmm_struct%dxyzqm(1,i)+ftmp*dx(3*i-2)
         qmmm_struct%dxyzqm(2,i)=qmmm_struct%dxyzqm(2,i)+ftmp*dx(3*i-1)
         qmmm_struct%dxyzqm(3,i)=qmmm_struct%dxyzqm(3,i)+ftmp*dx(3*i)
c$$$         if(master) then
c$$$            write(6,'(a6,1x,i3,4f15.8)') "master",i,dx(3*i-2),
c$$$     &           dx(3*i-1),dx(3*i)
c$$$         else
c$$$            write(*,'(a6,1x,i3,4f15.8)') "slave ",i,dx(3*i-2),
c$$$     &           dx(3*i-1),dx(3*i)
c$$$         end if
      end do
c
c     move qmmm_struct%dxyzqm to fmcmm
c     (link atoms are skipped)
c
      do i=1,qmmm_struct%nquant_nlink
         j=imcatm(i)
         if(qmmm_struct%mm_link_mask(j)) cycle
         fmcmm(3*j-2)=fmcmm(3*j-2)-qmmm_struct%dxyzqm(1,i)
         fmcmm(3*j-1)=fmcmm(3*j-1)-qmmm_struct%dxyzqm(2,i)
         fmcmm(3*j  )=fmcmm(3*j  )-qmmm_struct%dxyzqm(3,i)
      end do
c
c     For link atoms
c
      if(qmmm_struct%nlink.ne.0) then
c
c     Return the mm link pair coordinates back to the original mm atoms 
c     Needed before distributing link atom forces since it uses the unimaged array.
c
         call rst_mm_link_pair_crd(coords)
c
c     We need to divide the force on the link atom up between the
c     QM and MM atom.
c
         do i=1,qmmm_struct%nlink
            mm_no = 3*qmmm_struct%link_pairs(1,i)-2
            lnk_no = qmmm_struct%link_pairs(2,i)
            qm_no = 3*qmmm_nml%iqmatoms(lnk_no)-2
c
c     Note this routine uses the flink in the form -flink. 
c
            do j=1,qmmm_struct%nquant_nlink
               if(imcatm(j).eq.qmmm_struct%link_pairs(1,i)) then
                  dlink(1)=qmmm_struct%dxyzqm(1,j)
                  dlink(2)=qmmm_struct%dxyzqm(2,j)
                  dlink(3)=qmmm_struct%dxyzqm(3,j)
                  goto 100
               end if
            end do
c
 100        continue
c
            call distribute_lnk_f(frcmod,dlink,coords(mm_no),
     &                            coords(qm_no),qmmm_nml%lnk_dis)
c
c     NOTE: forces are reversed in QM calc with respect to amber force array
c     so we subtract forcemod from MM atom and add it to QM atom.
c     MM atom's new force = FMM(x,y,z) - FORCEMOD(x,y,z)
c
            fmcmm(mm_no)=fmcmm(mm_no)-frcmod(1)
            fmcmm(mm_no+1)=fmcmm(mm_no+1)-frcmod(2)
            fmcmm(mm_no+2)=fmcmm(mm_no+2)-frcmod(3)
c
c     QM atom's new force = FQM(x,y,z) - Flink(x,y,z) + FORCEMOD(x,y,z)
c     Note QM forces should be subtracted from sander F array to leave total force.
c
            fmcmm(qm_no)=fmcmm(qm_no)-dlink(1)+frcmod(1)
            fmcmm(qm_no+1)=fmcmm(qm_no+1)-dlink(2)+frcmod(2)
            fmcmm(qm_no+2)=fmcmm(qm_no+2)-dlink(3)+frcmod(3)
c
         end do
c
      end if
c
c     add frctmp to frc
c
      do i=1,3*natom
         frc(i)=frc(i)+fmcmm(i)/dble(numtasks)
      end do
c
c      call frccheck(frc,natom)
c
      return
      end
c     
c***************************************************************************
c     mccphi
c***************************************************************************
c     
      subroutine mccphi(coords,phi,natom,mcmatm,periodic)
c     
c     This subroutine calculates electrostatic potentials on QM atoms 
c     from MM regions 
c     
      use constants
      use nblist
      use qmmm_module
c     
      use mcinter
      use tapering
c     
      implicit double precision (a-h,o-z)
c     
      integer periodic
      dimension coords(3*natom)
      dimension phi(mcmatm)
c     
c
      if(periodic==0) goto 5000
c
c     periodic simulation
c
      if(itaper==1) then
c     
c     TINKER tapering function
c     
      do 1000 i=qmmm_mpi%nquant_nlink_start,qmmm_mpi%nquant_nlink_end
c     
         iq=imcatm(i)
         xiq=coords(3*iq-2)
         yiq=coords(3*iq-1)
         ziq=coords(3*iq  )
c
         jpair=0
c     
         do 1100 j=1,natom
c     
            if(qmmm_struct%atom_mask(j)) goto 1100
            if(qmmm_struct%mm_link_mask(j)) goto 1100
c     
            xj=coords(3*j-2)
            xij=xiq-xj
            xij=xij-a*anint(xij/a)
c     
            if(xij.gt.rcl) goto 1100
c     
            yj=coords(3*j-1)
            yij=yiq-yj
            yij=yij-b*anint(yij/b)
c     
            if(yij.gt.rcl) goto 1100
c     
            zj=coords(3*j)
            zij=ziq-zj
            zij=zij-c*anint(zij/c)
c     
            if(zij.gt.rcl) goto 1100
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c     
c     evaluate electrostatic potential using tapering function
c     
            if(delr .lt. rcl) then
c     
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c
               cj=qmmm_struct%scaled_mm_charges(j)
               b0=cj*delrinv
               dfee=b0*delr2inv
               b0=b0-cj*rch1
c     
               if(delr .ge. rcs) then
c     
                  delr3=delr2*delr
                  delr4=delr2*delr2
                  delr5=delr2*delr3
                  delr6=delr3*delr3
                  delr7=delr3*delr4
c     
                  taper=ct5*delr5+ct4*delr4+ct3*delr3
     &                 +ct2*delr2+ct1*delr+ct0
                  dtaper=5.0d+00*ct5*delr4+4.0d+00*ct4*delr3
     &                  +3.0d+00*ct3*delr2+2.0d+00*ct2*delr+ct1
                  trans=cj*(ft7*delr7+ft6*delr6+ft5*delr5
     &                 +ft4*delr4+ft3*delr3+ft2*delr2+ft1*delr+ft0)
                  dtrans=cj*(7.0d+00*ft7*delr6+6.0d+00*ft6*delr5
     &                  +5.0d+00*ft5*delr4+4.0d+00*ft4*delr3
     &                  +3.0d+00*ft3*delr2+2.0d+00*ft2*delr+ft1)
c
                  dfee=taper*dfee-(b0*dtaper+dtrans)*delrinv
                  b0=taper*b0+trans
c     
               end if
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 1'
               end if
c
               ipmcmm(jpair,i)=j
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            else
c     
               goto 1100
c     
            end if
c     
            phi(i)=phi(i)+b0
c     
 1100    continue
c
c
         if(iqmmmbd.eq.0) goto 1300
c
c     bond charge contribution for RC and RCD scheme
c
         do 1200 jbchg=1,nbchg
c
            jql=ibchgp(1,jbchg)
            jm2=ibchgp(2,jbchg)
c
            xm1=qmmm_struct%mm_link_pair_saved_coords(1,jql)
            ym1=qmmm_struct%mm_link_pair_saved_coords(2,jql)
            zm1=qmmm_struct%mm_link_pair_saved_coords(3,jql)
c
            xj=0.5d+00*(xm1+coords(3*jm2-2))
            xij=xiq-xj
            xij=xij-a*anint(xij/a)
c     
            if(xij.gt.rcl) goto 1200
c     
            yj=0.5d+00*(ym1+coords(3*jm2-1))
            yij=yiq-yj
            yij=yij-b*anint(yij/b)
c     
            if(yij.gt.rcl) goto 1200
c     
            zj=0.5d+00*(zm1+coords(3*jm2))
            zij=ziq-zj
            zij=zij-c*anint(zij/c)
c     
            if(zij.gt.rcl) goto 1200
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c
            if(delr.lt.rcl) then
c
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c
               cj=bndchg(jbchg)
               b0=cj*delrinv
               dfee=b0*delr2inv
               b0=b0-cj*rch1
c
               if(delr.ge.rcs) then
c
                  delr3 = delr2*delr
                  delr4 = delr2*delr2
                  delr5 = delr2*delr3
                  delr6 = delr3*delr3
                  delr7 = delr3*delr4
c
                  taper=ct5*delr5+ct4*delr4+ct3*delr3
     &                 +ct2*delr2+ct1*delr+ct0
                  dtaper=5.0d+00*ct5*delr4+4.0d+00*ct4*delr3
     &                  +3.0d+00*ct3*delr2+2.0d+00*ct2*delr+ct1
                  trans=cj*(ft7*delr7+ft6*delr6+ft5*delr5
     &                 +ft4*delr4+ft3*delr3+ft2*delr2+ft1*delr+ft0)
                  dtrans=cj*(7.0d+00*ft7*delr6+6.0d+00*ft6*delr5
     &                  +5.0d+00*ft5*delr4+4.0d+00*ft4*delr3
     &                  +3.0d+00*ft3*delr2+2.0d+00*ft2*delr+ft1)
c
                  dfee=taper*dfee-(b0*dtaper+dtrans)*delrinv
                  b0=taper*b0+trans
c
               end if
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 2'
               end if
c
               ipmcmm(jpair,i)=natom+jbchg
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            else
c     
               goto 1200
c     
            end if
c     
            phi(i)=phi(i)+b0
c     
 1200    continue
c
c
 1300    continue
c
         npmcmm(i)=jpair
c     
 1000 continue
c
c
      else 
c
c     usual electrostatic interaction
c
      do 1001 i=qmmm_mpi%nquant_nlink_start,qmmm_mpi%nquant_nlink_end
c     
         iq=imcatm(i)
         xiq=coords(3*iq-2)
         yiq=coords(3*iq-1)
         ziq=coords(3*iq  )
         jpair=0
c     
         do 1101 j=1,natom
c     
            if(qmmm_struct%atom_mask(j)) goto 1101
            if(qmmm_struct%mm_link_mask(j)) goto 1101
c     
            xj=coords(3*j-2)
            xij=xiq-xj
            xij=xij-a*anint(xij/a)
c     
            if(xij.gt.qmmm_nml%qmcut) goto 1101
c     
            yj=coords(3*j-1)
            yij=yiq-yj
            yij=yij-b*anint(yij/b)
c     
            if(yij.gt.qmmm_nml%qmcut) goto 1101
c     
            zj=coords(3*j)
            zij=ziq-zj
            zij=zij-c*anint(zij/c)
c     
            if(zij.gt.qmmm_nml%qmcut) goto 1101
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c     
c     evaluate electrostatic potential
c     
            if(delr .lt. qmmm_nml%qmcut) then
c     
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c            
               cj=qmmm_struct%scaled_mm_charges(j)
               b0=cj*delrinv
               phi(i)=phi(i)+b0
               dfee=b0*delr2inv
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 3'
               end if
c
               ipmcmm(jpair,i)=j
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            end if
c     
 1101    continue
c
c
         if(iqmmmbd.eq.0) goto 1301
c
c     bond charge contribution for RC and RCD scheme
c
         do 1201 jbchg=1,nbchg
c
            jql=ibchgp(1,jbchg)
            jm2=ibchgp(2,jbchg)
c
            xm1=qmmm_struct%mm_link_pair_saved_coords(1,jql)
            ym1=qmmm_struct%mm_link_pair_saved_coords(2,jql)
            zm1=qmmm_struct%mm_link_pair_saved_coords(3,jql)
c
            xj=0.5d+00*(xm1+coords(3*jm2-2))
            xij=xiq-xj
            xij=xij-a*anint(xij/a)
c     
            if(xij.gt.qmmm_nml%qmcut) goto 1201
c     
            yj=0.5d+00*(ym1+coords(3*jm2-1))
            yij=yiq-yj
            yij=yij-b*anint(yij/b)
c     
            if(yij.gt.qmmm_nml%qmcut) goto 1201
c     
            zj=0.5d+00*(zm1+coords(3*jm2))
            zij=ziq-zj
            zij=zij-c*anint(zij/c)
c     
            if(zij.gt.qmmm_nml%qmcut) goto 1201
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c
            if(delr.lt.qmmm_nml%qmcut) then
c
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c
               cj=bndchg(jbchg)
               b0=cj*delrinv
               phi(i)=phi(i)+b0
               dfee=b0*delr2inv
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 4'
               end if
c
               ipmcmm(jpair,i)=natom+jbchg
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            end if
c     
 1201    continue
c
 1301    continue
c
         npmcmm(i)=jpair
c     
 1001 continue
c
c
c
      end if
c     
c     unit to a.u.
c     
      do 300 i=qmmm_mpi%nquant_nlink_start,qmmm_mpi%nquant_nlink_end
         phi(i)=phi(i)*BOHRS_TO_A
c         write(6,*) i,phi(i)
 300  continue
c     
      return
c
c
c
 5000 continue
c
c     nonperiodic simulation
c
      if(itaper==1) then
c     
c     TINKER tapering function
c     
      do 1005 i=qmmm_mpi%nquant_nlink_start,qmmm_mpi%nquant_nlink_end
c     
         iq=imcatm(i)
         xiq=coords(3*iq-2)
         yiq=coords(3*iq-1)
         ziq=coords(3*iq  )
c
         jpair=0
c     
         do 1105 j=1,natom
c     
            if(qmmm_struct%atom_mask(j)) goto 1105
            if(qmmm_struct%mm_link_mask(j)) goto 1105
c     
            xj=coords(3*j-2)
            xij=xiq-xj
c     
            if(xij.gt.rcl) goto 1105
c     
            yj=coords(3*j-1)
            yij=yiq-yj
c     
            if(yij.gt.rcl) goto 1105
c     
            zj=coords(3*j)
            zij=ziq-zj
c     
            if(zij.gt.rcl) goto 1105
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c
            if(delr.lt.rcl) then
c
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c
               cj=qmmm_struct%scaled_mm_charges(j)
               b0=cj*delrinv
               dfee=b0*delr2inv
               b0=b0-cj*rch1
c
               if(delr.ge.rcs) then
c
                  delr3 = delr2*delr
                  delr4 = delr2*delr2
                  delr5 = delr2*delr3
                  delr6 = delr3*delr3
                  delr7 = delr3*delr4
c
                  taper=ct5*delr5+ct4*delr4+ct3*delr3
     &                 +ct2*delr2+ct1*delr+ct0
                  dtaper=5.0d+00*ct5*delr4+4.0d+00*ct4*delr3
     &                  +3.0d+00*ct3*delr2+2.0d+00*ct2*delr+ct1
                  trans=cj*(ft7*delr7+ft6*delr6+ft5*delr5
     &                 +ft4*delr4+ft3*delr3+ft2*delr2+ft1*delr+ft0)
                  dtrans=cj*(7.0d+00*ft7*delr6+6.0d+00*ft6*delr5
     &                  +5.0d+00*ft5*delr4+4.0d+00*ft4*delr3
     &                  +3.0d+00*ft3*delr2+2.0d+00*ft2*delr+ft1)
c
                  dfee=taper*dfee-(b0*dtaper+dtrans)*delrinv
                  b0=taper*b0+trans
c
               end if
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 5'
               end if
c
               ipmcmm(jpair,i)=j
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            else
c     
               goto 1105
c     
            end if
c     
            phi(i)=phi(i)+b0
c     
 1105    continue
c
c
         if(iqmmmbd.eq.0) goto 1305
c
c     bond charge contribution for RC and RCD scheme
c
         do 1205 jbchg=1,nbchg
c
            jql=ibchgp(1,jbchg)
            jm2=ibchgp(2,jbchg)
c
            xm1=qmmm_struct%mm_link_pair_saved_coords(1,jql)
            ym1=qmmm_struct%mm_link_pair_saved_coords(2,jql)
            zm1=qmmm_struct%mm_link_pair_saved_coords(3,jql)
c
            xj=0.5d+00*(xm1+coords(3*jm2-2))
            xij=xiq-xj
c     
            if(xij.gt.rcl) goto 1205
c     
            yj=0.5d+00*(ym1+coords(3*jm2-1))
            yij=yiq-yj
c     
            if(yij.gt.rcl) goto 1205
c     
            zj=0.5d+00*(zm1+coords(3*jm2))
            zij=ziq-zj
c     
            if(zij.gt.rcl) goto 1205
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c
            if(delr.lt.rcl) then
c
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c
               cj=bndchg(jbchg)
               b0=cj*delrinv
               dfee=b0*delr2inv
               b0=b0-cj*rch1
c
               if(delr.ge.rcs) then
c
                  delr3 = delr2*delr
                  delr4 = delr2*delr2
                  delr5 = delr2*delr3
                  delr6 = delr3*delr3
                  delr7 = delr3*delr4
c
                  taper=ct5*delr5+ct4*delr4+ct3*delr3
     &                 +ct2*delr2+ct1*delr+ct0
                  dtaper=5.0d+00*ct5*delr4+4.0d+00*ct4*delr3
     &                  +3.0d+00*ct3*delr2+2.0d+00*ct2*delr+ct1
                  trans=cj*(ft7*delr7+ft6*delr6+ft5*delr5
     &                 +ft4*delr4+ft3*delr3+ft2*delr2+ft1*delr+ft0)
                  dtrans=cj*(7.0d+00*ft7*delr6+6.0d+00*ft6*delr5
     &                  +5.0d+00*ft5*delr4+4.0d+00*ft4*delr3
     &                  +3.0d+00*ft3*delr2+2.0d+00*ft2*delr+ft1)
c
                  dfee=taper*dfee-(b0*dtaper+dtrans)*delrinv
                  b0=taper*b0+trans
c
               end if
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 6'
               end if
c
               ipmcmm(jpair,i)=natom+jbchg
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            else
c     
               goto 1205
c     
            end if
c     
            phi(i)=phi(i)+b0
c     
 1205    continue
c
c
 1305    continue
c
         npmcmm(i)=jpair
c     
 1005 continue
c
c
      else 
c
c     usual electrostatic interaction
c
      do 1006 i=qmmm_mpi%nquant_nlink_start,qmmm_mpi%nquant_nlink_end
c     
         iq=imcatm(i)
         xiq=coords(3*iq-2)
         yiq=coords(3*iq-1)
         ziq=coords(3*iq  )
         jpair=0
c     
         do 1106 j=1,natom
c     
            if(qmmm_struct%atom_mask(j)) goto 1106
            if(qmmm_struct%mm_link_mask(j)) goto 1106
c     
            xj=coords(3*j-2)
            xij=xiq-xj
c     
            if(xij.gt.qmmm_nml%qmcut) goto 1106
c     
            yj=coords(3*j-1)
            yij=yiq-yj
c     
            if(yij.gt.qmmm_nml%qmcut) goto 1106
c     
            zj=coords(3*j)
            zij=ziq-zj
c     
            if(zij.gt.qmmm_nml%qmcut) goto 1106
c     
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c     
c     evaluate electrostatic potential 
c     
            if(delr.lt.qmmm_nml%qmcut) then
c     
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c
               cj=qmmm_struct%scaled_mm_charges(j)
               b0=cj*delrinv
               dfee=b0*delr2inv
c
               phi(i)=phi(i)+b0
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 7'
               end if
c
               ipmcmm(jpair,i)=j
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            end if
c     
 1106    continue
c
         if(iqmmmbd.eq.0) goto 1306
c
c     bond charge contribution for RC and RCD scheme
c
         do 1206 jbchg=1,nbchg
c
            jql=ibchgp(1,jbchg)
            jm2=ibchgp(2,jbchg)
c
            xm1=qmmm_struct%mm_link_pair_saved_coords(1,jql)
            ym1=qmmm_struct%mm_link_pair_saved_coords(2,jql)
            zm1=qmmm_struct%mm_link_pair_saved_coords(3,jql)
c
            xj=0.5d+00*(xm1+coords(3*jm2-2))
            xij=xiq-xj
c     
            if(xij.gt.qmmm_nml%qmcut) goto 1206
c     
            yj=0.5d+00*(ym1+coords(3*jm2-1))
            yij=yiq-yj
c     
            if(yij.gt.qmmm_nml%qmcut) goto 1206
c     
            zj=0.5d+00*(zm1+coords(3*jm2))
            zij=ziq-zj
c     
            if(zij.gt.qmmm_nml%qmcut) goto 1206
c     
            delr2=xij*xij+yij*yij+zij*zij
            delr=sqrt(delr2)
c
            if(delr.lt.qmmm_nml%qmcut) then
c
               delrinv=one/delr
               delr2inv=delrinv*delrinv
c
               cj=bndchg(jbchg)
               b0=cj*delrinv
               phi(i)=phi(i)+b0
               dfee=b0*delr2inv
c
               dfx=dfee*xij
               dfy=dfee*yij
               dfz=dfee*zij
c     
c     store dfx, dfy, and dfz
c
               jpair=jpair+1
               if(jpair.gt.mxmmat) then
                  write(6,*) 'parameter mxmmat must be larger.'
                  stop 'mccphi 8'
               end if
c
               ipmcmm(jpair,i)=natom+jbchg
               dmcmmx(jpair,i)=dfx
               dmcmmy(jpair,i)=dfy
               dmcmmz(jpair,i)=dfz
c
            end if
c     
 1206    continue
c
 1306    continue
c
         npmcmm(i)=jpair
c     
 1006 continue
c
c
c
      end if
c     
c     unit to a.u.
c     
      do 305 i=qmmm_mpi%nquant_nlink_start,qmmm_mpi%nquant_nlink_end
         phi(i)=phi(i)*BOHRS_TO_A
c         write(6,*) i,phi(i)
 305  continue
c     
      return
c
      end
c
c ***************************************************************************
c     mcigrd
c ***************************************************************************
c
      subroutine mcigrd(coords,frc,natom,periodic)
c
c     This subroutine calculates electrostatic potentials on QM atoms 
c     from MM regions 
c
      use constants
      use nblist
      use qmmm_module
c
      use mcinter
      use tapering
c
      implicit double precision (a-h,o-z)
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      include 'eemcmm.inc'
c
      integer periodic
      dimension coords(3*natom),frc(3*natom)
c
c
c
      do 1000 i=qmmm_mpi%nquant_nlink_start,qmmm_mpi%nquant_nlink_end
c
         cgi=AMBER_ELECTROSTATIC2*chgr(i)
c
         do 1100 jpair=1,npmcmm(i)-nbchg
c
            j=ipmcmm(jpair,i)
c
            dfx=cgi*dmcmmx(jpair,i)
            dfy=cgi*dmcmmy(jpair,i)
            dfz=cgi*dmcmmz(jpair,i)
c
            qmmm_struct%dxyzqm(1,i)=qmmm_struct%dxyzqm(1,i)-dfx
            qmmm_struct%dxyzqm(2,i)=qmmm_struct%dxyzqm(2,i)-dfy
            qmmm_struct%dxyzqm(3,i)=qmmm_struct%dxyzqm(3,i)-dfz
c
            frc(3*j-2)=frc(3*j-2)-dfx
            frc(3*j-1)=frc(3*j-1)-dfy
            frc(3*j  )=frc(3*j  )-dfz
c
 1100    continue
c
c     bond charge contribution for RC and RCD scheme
c
         do 1200 jpair=npmcmm(i)-nbchg+1,npmcmm(i)
c
            jbchg=ipmcmm(jpair,i)-natom
c
            jql=ibchgp(1,jbchg)
            jm1=qmmm_nml%iqmatoms(qmmm_struct%nquant+jql)
            jm2=ibchgp(2,jbchg)
c
            dfx=cgi*dmcmmx(jpair,i)
            dfy=cgi*dmcmmy(jpair,i)
            dfz=cgi*dmcmmz(jpair,i)
c
            qmmm_struct%dxyzqm(1,i)=qmmm_struct%dxyzqm(1,i)-dfx
            qmmm_struct%dxyzqm(2,i)=qmmm_struct%dxyzqm(2,i)-dfy
            qmmm_struct%dxyzqm(3,i)=qmmm_struct%dxyzqm(3,i)-dfz
c
            dfxh=0.5d+00*dfx
            dfyh=0.5d+00*dfy
            dfzh=0.5d+00*dfz
c
            frc(3*jm1-2)=frc(3*jm1-2)-dfxh
            frc(3*jm1-1)=frc(3*jm1-1)-dfyh
            frc(3*jm1  )=frc(3*jm1  )-dfzh
c
            frc(3*jm2-2)=frc(3*jm2-2)-dfxh
            frc(3*jm2-1)=frc(3*jm2-1)-dfyh
            frc(3*jm2  )=frc(3*jm2  )-dfzh
c
 1200    continue
c
 1000 continue
c
      return
c
      end
c
c ***************************************************************************
c     mcpchg
c ***************************************************************************
      subroutine mcpchg
c
c     Subroutine to output EE-MCMM charges
c
      use mcinter
c
      implicit double precision (a-h,o-z)
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      include 'eemcmm.inc'
c
      if(ifmtchg.eq.0) then
c
c     unformatted
c
         write(fu6,*) 'not programmed yet'
         stop
c
      else
c
c     formatted
c
         write(iochg,9999) (chgr(i),i=1,mcnatm)
c
      end if
c
      return
c
 9999 format(6f12.7)
c
      end
c
c ***************************************************************************
c     mcpelp
c ***************************************************************************
      subroutine mcpelp
c
c     Subroutine to output electrostatic potential on each QM atom
c
      use mcinter
c
      implicit double precision (a-h,o-z)
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      include 'eemcmm.inc'
c
      if(ifmtelp.eq.0) then
c
c     unformatted
c
         write(fu6,*) 'not programmed yet'
         stop
c
      else
c
c     formatted
c
         write(ioelp,9999) (phir(i),i=1,mcnatm)
c
      end if
c
      return
c
 9999 format(6f12.7)
c
      end
c
c ***************************************************************************
c     mcpene
c ***************************************************************************
      subroutine mcpene(nstep)
c
c     Subroutine to output hamiltonian matrix and (EE-)MCMM energy
c
      use mcinter
      use constants
c
      implicit double precision (a-h,o-z)
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      include 'eemcmm.inc'
c
      if(mciee.eq.1) then
c
c     calculate EE-MCMM QM/MM interaction energy
c
         vqmmm=0.0d+00
c
         do 100 i=1,mcnatm
c
            vqmmm=vqmmm+chgr(i)*phir(i)
c
 100     continue
c
      end if
c
c     unit from a.u. to kcal/mol
c
      v11k=AU_TO_KCAL*vmc11
      v22k=AU_TO_KCAL*vmc22
      if(icomplex.eq.1) then
         if(vmc12.ge.0.0d+00) then
            v12k=AU_TO_KCAL*sqrt(vmc12)         
         else
            v12k=-AU_TO_KCAL*sqrt(-vmc12)         
         end if
      else
         v12k=AU_TO_KCAL*vmc12         
      end if
      veigk=AU_TO_KCAL*vmceig
      vqmmmk=AU_TO_KCAL*vqmmm
c
c     output
c
      if(ifmtelp.eq.0) then
c
c     unformatted
c
         write(fu6,*) 'not programmed yet'
         stop
c
      else
c
c     formatted
c
         if(mciee.eq.1) then
            write(ioene,9999) nstep,v11k,v22k,v12k,veigk,vqmmmk
         else
            write(ioene,9999) nstep,v11k,v22k,v12k,veigk
         end if
c
      end if
c
      return
c
 9999 format(1x,i7,5f12.5)
c
      end
c
c ***************************************************************************
c     mcrsml
c ***************************************************************************
c
      subroutine mcrsml(string,iend,istrt)
c
c     Subroutine to read in the input the SIMULATION section of MC-TINKER.
c
      use qmmm_module
      use mcinter
c
      implicit double precision (a-h,o-z)
c
#ifdef MPI
c     =========================== AMBER/MPI ===========================
#     ifdef MPI_DOUBLE_PRECISION
#       undef MPI_DOUBLE_PRECISION
#     endif
      include 'mpif.h'
#     ifdef CRAY_PVP
#       define MPI_DOUBLE_PRECISION MPI_REAL8
#     endif
c     ========================= END AMBER/MPI =========================
#endif
c
#     include "parallel.h"
#     include "extra.h"
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      character*80 string
      character*3 endwrd
      character*5 bound(5)
c
      data bound/'AMBER','RC   ','RCD  ','RC2  ' ,'RCD2 '/
c
c     set default values
c
      nstpchg=0
      nstpelp=0
      nstpene=0
      ifmtchg=1
      ifmtelp=1
      ifmtene=1
      iqmmmbd=0
c
c      do 100 i=1,qmmm_struct%nquant_nlink
c         imcatm(i)=qmmm_nml%iqmatoms(i)
c 100  continue
c
c     read in first keyword in this section
c
      call mcrlin(fu5,string,istrt,isect,iend)
c
      do while (isect.eq.0.and.iend.eq.0)
c
c NSTPCHG
c
         if (string(istrt:istrt+6).eq.'nstpchg') then
            call rword(string,istrt,ierr)
            if (ierr.ne.1) then
               nstpchg = icmct(string(istrt:80))
            else
               write(fu6,1000)
               stop 'mcrsml 1'
            end if
 1000 format (1x,'Error: Keyword NSTPCHG must have an argument.')
c
c NSTPELP
c
         else if (string(istrt:istrt+6).eq.'nstpelp') then
            call rword(string,istrt,ierr)
            if (ierr.ne.1) then
               nstpelp = icmct(string(istrt:80))
            else
               write(fu6,1100)
               stop 'mcrsml 2'
            end if
 1100 format (1x,'Error: Keyword NSTPELP must have an argument.')
c
c NSTPENE
c
         else if (string(istrt:istrt+6).eq.'nstpene') then
            call rword(string,istrt,ierr)
            if (ierr.ne.1) then
               nstpene = icmct(string(istrt:80))
            else
               write(fu6,1200)
               stop 'mcrsml 3'
            end if
 1200 format (1x,'Error: Keyword NSTPENE must have an argument.')
c
c FMTCHG
c
         else if (string(istrt:istrt+5).eq.'fmtchg') then
            call rword(string,istrt,ierr)
            if (ierr.ne.1) then
               if (string(istrt:istrt+8).eq.'formatted') then
                  ifmtchg = 1
               elseif (string(istrt:istrt+5).eq.'binary') then
                  ifmtchg = 0
               else
                  write(fu6,1300)
                  stop 'mcrsml 4'
               endif
            else
               write(fu6,1310)
               stop 'mcrsml 5'
            endif
 1300 format (1x,'Error: Keyword FMTCHG has a wrong argument.')
 1310 format (1x,'Error: Keyword FMTCHG must have an argument.')
c
c FMTELP
c
         else if (string(istrt:istrt+5).eq.'fmtelp') then
            call rword(string,istrt,ierr)
            if (ierr.ne.1) then
               if (string(istrt:istrt+8).eq.'formatted') then
                  ifmtelp = 1
               elseif (string(istrt:istrt+5).eq.'binary') then
                  ifmtelp = 0
               else
                  write(fu6,1400)
                  stop 'mcrsml 6'
               endif
            else
               write(fu6,1410)
               stop 'mcrsml 7'
            endif
 1400 format (1x,'Error: Keyword FMTELP has a wrong argument.')
 1410 format (1x,'Error: Keyword FMTELP must have an argument.')
c
c FMTENE
c
         else if (string(istrt:istrt+5).eq.'fmtene') then
            call rword(string,istrt,ierr)
            if (ierr.ne.1) then
               if (string(istrt:istrt+8).eq.'formatted') then
                  ifmtelp = 1
               elseif (string(istrt:istrt+5).eq.'binary') then
                  ifmtelp = 0
               else
                  write(fu6,1500)
                  stop 'mcrsml 8'
               endif
            else
               write(fu6,1510)
               stop 'mcrsml 9'
            endif
 1500 format (1x,'Error: Keyword FMTENE has a wrong argument.')
 1510 format (1x,'Error: Keyword FMTENE must have an argument.')
c
c IMCMMATOM or IMCSIATOM
c
         else if ((string(istrt:istrt+8).eq.'imcmmatom').or.
     &            (string(istrt:istrt+8).eq.'imcsiatom')) then
            if(master) then
               do 200 i=1,qmmm_struct%nquant_nlink
                  read(fu5,*) imcatm(i)
 200           continue
               read(fu5,*) endwrd
               if(endwrd.ne.'END' .and. endwrd.ne.'end') then
                  write(fu6,1600)
                  stop 'mcrsml 10'
               end if
            end if
#ifdef MPI
            call mpi_bcast(imcatm, qmmm_struct%nquant_nlink, 
     &           mpi_integer, 0, commsander, ier) 
#endif
 1600 format (1x,'Error: Keyword IMCMMATOM must be an array of QM atoms',/
     &        1x,'(including link atoms)' )
c
c BOUNDARY
c
         else if (string(istrt:istrt+7).eq.'boundary') then
            call rword(string,istrt,ierr)
            if (ierr.ne.1) then
               if (string(istrt:istrt+4).eq.'amber') then
                  iqmmmbd = 0
               else if (string(istrt:istrt+2).eq.'rc ') then
                  iqmmmbd = 1
               else if (string(istrt:istrt+3).eq.'rcd ') then
                  iqmmmbd = 2
               else if (string(istrt:istrt+3).eq.'brc ') then
                  iqmmmbd = 3
               else if (string(istrt:istrt+3).eq.'brcd') then
                  iqmmmbd = 4
               else
                  write(fu6,1700)
                  stop 'mcrsml 12'
               endif
            else
               write(fu6,1710)
               stop 'mcrsml 13'
            endif
 1700 format (1x,'Error: Keyword BOUNDARY has a wrong argument.')
 1710 format (1x,'Error: Keyword BOUNDARY must have an argument.')
c
         else
            write(fu6,2000) string(istrt:80)
            stop 'mcrsml 14'
         end if
 2000 format(1x,'Error: the following is not a valid keyword in ',
     *          'the SIMULATION section:',/A80)
c
         call mcrlin(fu5,string,istrt,isect,iend)
      end do
c
c     output
c
      if(master) then
         write(fu6,9999)
         write(fu6,9998) nstpene,nstpchg,nstpelp,bound(iqmmmbd+1)
c
         write(fu6,9997)
         do 300 i=1,qmmm_struct%nquant_nlink
            write(fu6,9996) i, imcatm(i)
 300     continue
      end if
c
c
      return
c
 9999 format(/10x,'SIMULARTION Section',/)
 9998 format(1x,'NSTPENE=',I4,5X,'NSTPCHG=',I4,5X,'NSTPELP=',I4,5X,
     &      'BOUNDARY=',A5)
 9997 format(/5x,'QM(MCMM) No.   MM No.')        
 9996 format(8x,i6,5x,i6)
c
      end
c
c ***************************************************************************
c     mcfset
c ***************************************************************************
c
      subroutine mcfset(iunitr,finame)
c
c     This subroutine sets up file name according to iunitr.
c     This subroutine is replaced with another one
c     when mc-tinker is combined with sander.
c
      use mcinter
c
      implicit real*8 (a-h,o-z)
      character*256 finame
c
      if(iunitr .eq. 81) then
         finame = esp81
      else if (iunitr .eq. 82) then
         finame = esp82
      else if (iunitr .eq. 83) then
         finame = esp83
      else if (iunitr. eq. 72) then
         finame = 'esp.fu72'
      else if (iunitr. eq. 73) then
         finame = 'esp.fu73'
      else if (iunitr. eq. 74) then
         finame = 'esp.fu74'
      else if (iunitr. eq. 75) then
         finame = 'esp.fu75'
      else if (iunitr. eq. 76) then
         finame = 'esp.fu76'
      else if (iunitr. eq. 62) then
         finame = 'esp.fu62'
      else if (iunitr. eq. 63) then
         finame = 'esp.fu63'
      else if (iunitr. eq. 64) then
         finame = 'esp.fu64'
      else if (iunitr. eq. 65) then
         finame = 'esp.fu65'
      else if (iunitr. eq. 66) then
         finame = 'esp.fu66'
      end if
c
      return
      end
c
c ***************************************************************************
c     mcsrcd
c ***************************************************************************
c
      subroutine mcsrcd(nbona,iba,jba,nbonh,ibh,jbh,natom,chg,
     &                  ipres,labres,nres)
c
c     set up for the RC and RCD method
c
      use mcinter
      use qmmm_module
      use constants
c
      implicit real*8 (a-h,o-z)
c     
      dimension iba(nbona),jba(nbona)
      dimension ibh(nbonh),jbh(nbonh)
      dimension chg(natom),ipres(nres)
      dimension ilres(qmmm_struct%nlink),ildvq(qmmm_struct%nlink)
      character*4 labres(nres)
      character*5 bound(5)
c
      data bound/'AMBER','RC   ','RCD  ','BRC  ' ,'BRCD '/
c
c
      write(6,9000)
      if(iqmmmbd.eq.1) write(6,9101)
      if(iqmmmbd.eq.2) write(6,9102)
      if(iqmmmbd.eq.3) write(6,9103)
      if(iqmmmbd.eq.4) write(6,9104)
c
      if(iqmmmbd.le.2) goto 2000
c
c     ----- charge conservation for balanced rc or rcd scheme -----
c
      do 101 il=1,qmmm_struct%nlink
c
         im1=qmmm_nml%iqmatoms(qmmm_struct%nquant+il)
         ildvq(il)=1
c
         do 111 ires=1,nres
            do 121 ia=ipres(ires),ipres(ires+1)-1
               if(ia.eq.im1) then
                  ilres(il)=ires
                  goto 131
               end if
 121        continue
 111     continue
c
 131     continue
         do 141 jl=1,il-1
            if(ilres(il).eq.ilres(jl)) then
               ildvq(il)=ildvq(il)+1
               ildvq(jl)=ildvq(jl)+1
            end if
 141     continue
c
 101  continue
C
      write(6,9200)
      do 201 il=1,qmmm_struct%nlink
         chgmm=zero
         im1=qmmm_nml%iqmatoms(qmmm_struct%nquant+il)
         ires=ilres(il)
         is=ipres(ires)
         ie=ipres(ires+1)-1
         do 211 i=1,qmmm_struct%nquant
            im=qmmm_nml%iqmatoms(i)
            if((is.le.im).and.(im.le.ie)) then
               chgmm=chgmm+chg(im)
            end if
 211     continue
         chgmm=chgmm*INV_AMBER_ELECTROSTATIC
         idiv=ildvq(il)
         ichgqm=nint(chgmm)
         cm1old=chg(im1)*INV_AMBER_ELECTROSTATIC
         chg(im1)=chg(im1)
     &           +AMBER_ELECTROSTATIC*(chgmm-dble(ichgqm))/dble(idiv)
c
         write(6,9201) il,im1,labres(ires),ires,
     &                 chgmm,ichgqm,cm1old,
     &                 chg(im1)*INV_AMBER_ELECTROSTATIC
c
 201  continue
C
C
 2000 continue
C
      do 11 iat=1,qmmm_struct%nquant
         im=qmmm_nml%iqmatoms(iat)
         chg(im)=zero
 11   continue
c
      do 21 i=1,natom
         qmmm_struct%scaled_mm_charges(i)=chg(i)*INV_AMBER_ELECTROSTATIC
     &                                  *qmmm_nml%chg_lambda
 21   continue
c
c      
      if((iqmmmbd.eq.1).or.(iqmmmbd.eq.3))then
         write(6,9300)
      else
         write(6,9310)
      end if
c
      nbchg=0
      do 1001 il=1,qmmm_struct%nlink
c
         im1=qmmm_nml%iqmatoms(qmmm_struct%nquant+il)
         qmmm_struct%scaled_mm_charges(im1)=zero
         nm2atm=0
c
         do 100 ibona=1,nbona
            ib=iba(ibona)/3+1
            jb=jba(ibona)/3+1
            if(jb.eq.im1) then
               if(.not.qmmm_struct%atom_mask(ib)) then
                  nm2atm=nm2atm+1
                  nbchg=nbchg+1
                  ibchgp(1,nbchg)=il
                  ibchgp(2,nbchg)=ib
               end if
            end if
            if(ib.eq.im1) then
               if(.not.qmmm_struct%atom_mask(jb)) then
                  nm2atm=nm2atm+1
                  nbchg=nbchg+1
                  ibchgp(1,nbchg)=il
                  ibchgp(2,nbchg)=jb
               end if
            end if
 100     continue
c
         do 200 ibonh=1,nbonh
            ib=ibh(ibonh)/3+1
            jb=jbh(ibonh)/3+1
            if(jb.eq.im1) then
               if(.not.qmmm_struct%atom_mask(ib)) then
                  nm2atm=nm2atm+1
                  nbchg=nbchg+1
                  ibchgp(1,nbchg)=il
                  ibchgp(2,nbchg)=ib
               end if
            end if
            if(ib.eq.im1) then
               if(.not.qmmm_struct%atom_mask(jb)) then
                  nm2atm=nm2atm+1
                  nbchg=nbchg+1
                  ibchgp(1,nbchg)=il
                  ibchgp(2,nbchg)=jb
               end if
            end if
 200     continue
c
         dstchg=INV_AMBER_ELECTROSTATIC*chg(im1)/dble(nm2atm)
c
         if((iqmmmbd.eq.1).or.(iqmmmbd.eq.3)) then
c
c     ----- RC scheme -----
c
            do 401 inm2=1,nm2atm
               idxim2=nbchg+inm2-nm2atm
               bndchg(idxim2)=dstchg
               im2=ibchgp(2,idxim2)
               if(inm2.eq.1) then
                  iqm=qmmm_struct%link_pairs(2,qmmm_struct%nlink)
                  iq1=qmmm_nml%iqmatoms(iqm)
                  write(6,9301) il,iq1,im1,im2,
     &            INV_AMBER_ELECTROSTATIC*chg(im1),dstchg
               else
                  write(6,9302) im2,dstchg
               end if
 401        continue
c
         else
c
c     ----- RCD scheme -----
c
            do 402 inm2=1,nm2atm
               idxim2=nbchg+inm2-nm2atm
               bndchg(idxim2)=2.0d+00*dstchg
               ibtmp=ibchgp(2,idxim2)
               qmmm_struct%scaled_mm_charges(ibtmp)
     &        =qmmm_struct%scaled_mm_charges(ibtmp)-dstchg
               im2=ibchgp(2,idxim2)
               if(inm2.eq.1) then
                  iqm=qmmm_struct%link_pairs(2,qmmm_struct%nlink)
                  iq1=qmmm_nml%iqmatoms(iqm)
                  write(6,9311) il,iq1,im1,im2,
     &                 INV_AMBER_ELECTROSTATIC*chg(im1),
     &                 INV_AMBER_ELECTROSTATIC*chg(im2),
     &            2.0d+00*dstchg,qmmm_struct%scaled_mm_charges(ibtmp)
               else
                  write(6,9312) im2,INV_AMBER_ELECTROSTATIC*chg(im2),
     &            2.0d+00*dstchg,qmmm_struct%scaled_mm_charges(ibtmp)
               end if
 402        continue
c
         end if
c
 1001 continue
c
      write(6,9400)
c
      return
c
 9000 format(/10x,'----- QM-MM Boundary -----')
 9101 format(/5x,'Redistributed-charge scheme was used.')
 9102 format(/5x,'Redistributed-charge-dipole scheme was used.')
 9103 format(/5x,'Balanced redistributed-charge scheme was used.')
 9104 format(/5x,'Balanced redistributed-charge-dipole scheme was ',
     &        'used.')
 9200 format(/6x,'No.',3X,'M1',5X,'Residue',5x,'MMSUM',3x,'QMSUM',3x,
     &       'Qold(M1)'2x,'Qnew(M1)',/5x,60(1h-)) 
 9201 format(5x,i3,1x,i6,4x,a4,i4,2x,f8.4,3x,i3,3x,f8.4,2x,f8.4)
 9300 format(/6x,'No.',3x,'Q1',5x,'M1',5x,'M2',5x,'Qorg(M1)',2x,
     &       'QRC(M1-M2)',/5x,48(1h-))
 9301 format(5x,i3,1x,i6,1x,i6,1x,i6,3x,f8.4,3x,f8.4)
 9302 format(23x,i6,14x,f8.4)
 9310 format(/6x,'No.',3x,'Q1',5x,'M1',5x,'M2',5x,'Qorg(M1)',3x,
     &       'Qorg(M2)'1x,'QRCD(M1-M2)',2x,'Qrcd(M2)',/5x,69(1h-))
 9311 format(5x,i3,1x,i6,1x,i6,1x,i6,4(3x,f8.4))
 9312 format(23x,i6,11x,3(3x,f8.4))
 9400 format(/80('-')/)
c
      end
c
c     subroutine to check forces
c
      subroutine frccheck(frc,natom)
c
c     This subroutine is the driver for the actual MCMM calculation.
c     (Titus V Albu January 2002)
c
      use qmmm_module
      use constants
      use mcinter
c
      implicit double precision (a-h,o-z)
c
      include 'mcparm.inc'
      include 'mcperc.inc'
      include 'mccomm.inc'
c
      include 'eemcmm.inc'                                              0831MH07 
c
      dimension frc(3*natom)
c
      ftmp=AU_TO_KCAL/BOHRS_TO_A
c
      write(6,*) "in frccheck"
c
c$$$      do i=1,qmmm_struct%nquant_nlink
c$$$         j=imcatm(i)
c$$$         write(6,'(1x,i3,4f15.8)') i,
c$$$     &        frc(3*j-2)/ftmp,frc(3*j-1)/ftmp,frc(3*j)/ftmp
c$$$      end do
c
      do i=1,natom
         write(6,'(1x,i6,4f15.8)') i,
     &        frc(3*i-2)/ftmp,frc(3*i-1)/ftmp,frc(3*i)/ftmp
      end do
c
      return
      end
